@@ -70,6 +70,7 @@
 
 /* we include this here to get the G_OS_* defines */
 #include <glib.h>
+#include <gst/gst.h>
 
 #ifdef G_OS_WIN32
 #include <winsock2.h>
@@ -109,9 +110,11 @@ G_STMT_START {                                  \
 } G_STMT_END
 
 #ifdef G_OS_WIN32
+#define FIONREAD_TYPE gulong
 #define IOCTL_SOCKET ioctlsocket
 #define CLOSE_SOCKET(sock) closesocket(sock);
 #else
+#define FIONREAD_TYPE gint
 #define IOCTL_SOCKET ioctl
 #define CLOSE_SOCKET(sock) close(sock);
 #endif
@@ -160,7 +163,7 @@ gst_rtsp_connection_create (GstRTSPUrl * url, GstRTSPConnection ** conn)
 #ifdef G_OS_WIN32
   /* This should work on UNIX too. PF_UNIX sockets replaced with pipe */
   /* pipe( CONTROL_SOCKETS(newconn) ) */
-  if ((ret = pipe (CONTROL_SOCKETS (newconn))) < 0)
+  if ((ret = _pipe (CONTROL_SOCKETS (newconn), 4096, _O_BINARY)) < 0)
     goto no_socket_pair;
 
   ioctlsocket (READ_SOCKET (newconn), FIONBIO, &flags);
@@ -212,9 +215,7 @@ gst_rtsp_connection_connect (GstRTSPConnection * conn, GTimeVal * timeout)
   gint fd;
   struct sockaddr_in sa_in;
   struct hostent *hostinfo;
-  char **addrs;
   const gchar *ip;
-  gchar ipbuf[INET_ADDRSTRLEN];
   struct in_addr addr;
   gint ret;
   guint16 port;
@@ -226,6 +227,10 @@ gst_rtsp_connection_connect (GstRTSPConnection * conn, GTimeVal * timeout)
 
 #ifdef G_OS_WIN32
   unsigned long flags;
+  struct in_addr *addrp;
+#else
+  char **addrs;
+  gchar ipbuf[INET_ADDRSTRLEN];
 #endif /* G_OS_WIN32 */
 
   g_return_val_if_fail (conn != NULL, GST_RTSP_EINVAL);
@@ -244,10 +249,15 @@ gst_rtsp_connection_connect (GstRTSPConnection * conn, GTimeVal * timeout)
 
     if (hostinfo->h_addrtype != AF_INET)
       goto not_ip;              /* host not an IP host */
-
+#ifdef G_OS_WIN32
+    addrp = (struct in_addr *) hostinfo->h_addr_list[0];
+    /* this is not threadsafe */
+    ip = inet_ntoa (*addrp);
+#else
     addrs = hostinfo->h_addr_list;
     ip = inet_ntop (AF_INET, (struct in_addr *) addrs[0], ipbuf,
         sizeof (ipbuf));
+#endif /* G_OS_WIN32 */
   }
 
   /* get the port from the url */
@@ -283,7 +293,7 @@ gst_rtsp_connection_connect (GstRTSPConnection * conn, GTimeVal * timeout)
   FD_ZERO (&readfds);
   FD_SET (READ_SOCKET (conn), &readfds);
 
-  if (timeout->tv_sec != 0 || timeout->tv_usec != 0) {
+  if (timeout) {
     tv.tv_sec = timeout->tv_sec;
     tv.tv_usec = timeout->tv_usec;
     tvp = &tv;
@@ -398,7 +408,6 @@ gst_rtsp_connection_write (GstRTSPConnection * conn, const guint8 * data,
   FD_ZERO (&writefds);
   FD_SET (conn->fd, &writefds);
   FD_ZERO (&readfds);
-  FD_SET (READ_SOCKET (conn), &readfds);
 
   max_fd = MAX (conn->fd, READ_SOCKET (conn));
 
@@ -415,6 +424,9 @@ gst_rtsp_connection_write (GstRTSPConnection * conn, const guint8 * data,
   while (towrite > 0) {
     gint written;
 
+    /* set bit inside the loop for when we loop to read the rest of the data */
+    FD_SET (READ_SOCKET (conn), &readfds);
+
     do {
       retval = select (max_fd + 1, &readfds, &writefds, NULL, tvp);
     } while ((retval == -1 && errno == EINTR));
@@ -425,20 +437,8 @@ gst_rtsp_connection_write (GstRTSPConnection * conn, const guint8 * data,
     if (retval == -1)
       goto select_error;
 
-    if (FD_ISSET (READ_SOCKET (conn), &readfds)) {
-      /* read all stop commands */
-      while (TRUE) {
-        gchar command;
-        int res;
-
-        READ_COMMAND (conn, command, res);
-        if (res <= 0) {
-          /* no more commands */
-          break;
-        }
-      }
+    if (FD_ISSET (READ_SOCKET (conn), &readfds))
       goto stopped;
-    }
 
     /* now we can write */
     written = write (conn->fd, data, towrite);
@@ -550,7 +550,7 @@ gst_rtsp_connection_send (GstRTSPConnection * conn, GstRTSPMessage * message,
       break;
     }
     default:
-      g_assert_not_reached ();
+      g_return_val_if_reached (GST_RTSP_EINVAL);
       break;
   }
 
@@ -787,34 +787,31 @@ no_column:
 }
 
 /**
- * gst_rtsp_connection_read:
+ * gst_rtsp_connection_read_internal:
  * @conn: a #GstRTSPConnection
  * @data: the data to read
  * @size: the size of @data
  * @timeout: a timeout value or #NULL
+ * @allow_interrupt: can the pending read be interrupted
  *
  * Attempt to read @size bytes into @data from the connected @conn, blocking up to
  * the specified @timeout. @timeout can be #NULL, in which case this function
  * might block forever.
  * 
- * This function can be canceled with gst_rtsp_connection_flush().
+ * This function can be canceled with gst_rtsp_connection_flush() only if the
+ * @allow_interrupt is set.
  *
  * Returns: #GST_RTSP_OK on success.
  */
-GstRTSPResult
-gst_rtsp_connection_read (GstRTSPConnection * conn, guint8 * data, guint size,
-    GTimeVal * timeout)
+static GstRTSPResult
+gst_rtsp_connection_read_internal (GstRTSPConnection * conn, guint8 * data,
+    guint size, GTimeVal * timeout, gboolean allow_interrupt)
 {
   fd_set readfds;
   guint toread;
   gint retval;
-  struct timeval tv_timeout, *ptv_timeout = NULL;
-
-#ifndef G_OS_WIN32
-  gint avail;
-#else
-  gulong avail;
-#endif
+  struct timeval tv_timeout, *ptv_timeout;
+  FIONREAD_TYPE avail;
 
   g_return_val_if_fail (conn != NULL, GST_RTSP_EINVAL);
   g_return_val_if_fail (data != NULL, GST_RTSP_EINVAL);
@@ -824,6 +821,14 @@ gst_rtsp_connection_read (GstRTSPConnection * conn, guint8 * data, guint size,
 
   toread = size;
 
+  /* configure timeout if any */
+  if (timeout != NULL) {
+    tv_timeout.tv_sec = timeout->tv_sec;
+    tv_timeout.tv_usec = timeout->tv_usec;
+    ptv_timeout = &tv_timeout;
+  } else
+    ptv_timeout = NULL;
+
   /* if the call fails, just go in the select.. it should not fail. Else if
    * there is enough data to read, skip the select call al together.*/
   if (IOCTL_SOCKET (conn->fd, FIONREAD, &avail) < 0)
@@ -831,20 +836,17 @@ gst_rtsp_connection_read (GstRTSPConnection * conn, guint8 * data, guint size,
   else if (avail >= toread)
     goto do_read;
 
-  /* configure timeout if any */
-  if (timeout != NULL) {
-    tv_timeout.tv_sec = timeout->tv_sec;
-    tv_timeout.tv_usec = timeout->tv_usec;
-    ptv_timeout = &tv_timeout;
-  }
-
   FD_ZERO (&readfds);
   FD_SET (conn->fd, &readfds);
-  FD_SET (READ_SOCKET (conn), &readfds);
 
   while (toread > 0) {
     gint bytes;
 
+    /* set inside the loop so that when we did not read enough and we have to
+     * continue, we still have the cancel socket bit set */
+    if (allow_interrupt)
+      FD_SET (READ_SOCKET (conn), &readfds);
+
     do {
       retval = select (FD_SETSIZE, &readfds, NULL, NULL, ptv_timeout);
     } while ((retval == -1 && errno == EINTR));
@@ -856,20 +858,8 @@ gst_rtsp_connection_read (GstRTSPConnection * conn, guint8 * data, guint size,
     if (retval == 0)
       goto select_timeout;
 
-    if (FD_ISSET (READ_SOCKET (conn), &readfds)) {
-      /* read all stop commands */
-      while (TRUE) {
-        gchar command;
-        int res;
-
-        READ_COMMAND (conn, command, res);
-        if (res <= 0) {
-          /* no more commands */
-          break;
-        }
-      }
+    if (FD_ISSET (READ_SOCKET (conn), &readfds))
       goto stopped;
-    }
 
   do_read:
     /* if we get here there is activity on the real fd since the select
@@ -911,6 +901,29 @@ read_error:
   }
 }
 
+/**
+ * gst_rtsp_connection_read:
+ * @conn: a #GstRTSPConnection
+ * @data: the data to read
+ * @size: the size of @data
+ * @timeout: a timeout value or #NULL
+ *
+ * Attempt to read @size bytes into @data from the connected @conn, blocking up to
+ * the specified @timeout. @timeout can be #NULL, in which case this function
+ * might block forever.
+ *
+ * This function can be canceled with gst_rtsp_connection_flush().
+ *
+ * Returns: #GST_RTSP_OK on success.
+ */
+GstRTSPResult
+gst_rtsp_connection_read (GstRTSPConnection * conn, guint8 * data, guint size,
+    GTimeVal * timeout)
+{
+  return gst_rtsp_connection_read_internal (conn, data, size, timeout, TRUE);
+}
+
+
 static GstRTSPResult
 read_body (GstRTSPConnection * conn, glong content_length, GstRTSPMessage * msg,
     GTimeVal * timeout)
@@ -927,8 +940,8 @@ read_body (GstRTSPConnection * conn, glong content_length, GstRTSPMessage * msg,
   body = g_malloc (content_length + 1);
   body[content_length] = '\0';
 
-  GST_RTSP_CHECK (gst_rtsp_connection_read (conn, body, content_length,
-          timeout), read_error);
+  GST_RTSP_CHECK (gst_rtsp_connection_read_internal (conn, body, content_length,
+          timeout, FALSE), read_error);
 
   content_length += 1;
 
@@ -982,8 +995,9 @@ gst_rtsp_connection_receive (GstRTSPConnection * conn, GstRTSPMessage * message,
     guint8 c;
 
     /* read first character, this identifies data messages */
-    GST_RTSP_CHECK (gst_rtsp_connection_read (conn, &c, 1, timeout),
-        read_error);
+    /* This is the only read() that we allow to be interrupted */
+    GST_RTSP_CHECK (gst_rtsp_connection_read_internal (conn, &c, 1, timeout,
+            TRUE), read_error);
 
     /* check for data packet, first character is $ */
     if (c == '$') {
@@ -992,15 +1006,15 @@ gst_rtsp_connection_receive (GstRTSPConnection * conn, GstRTSPMessage * message,
       /* data packets are $<1 byte channel><2 bytes length,BE><data bytes> */
 
       /* read channel, which is the next char */
-      GST_RTSP_CHECK (gst_rtsp_connection_read (conn, &c, 1, timeout),
-          read_error);
+      GST_RTSP_CHECK (gst_rtsp_connection_read_internal (conn, &c, 1, timeout,
+              FALSE), read_error);
 
       /* now we create a data message */
       gst_rtsp_message_init_data (message, c);
 
       /* next two bytes are the length of the data */
-      GST_RTSP_CHECK (gst_rtsp_connection_read (conn, (guint8 *) & size, 2,
-              timeout), read_error);
+      GST_RTSP_CHECK (gst_rtsp_connection_read_internal (conn,
+              (guint8 *) & size, 2, timeout, FALSE), read_error);
 
       size = GUINT16_FROM_BE (size);
 
@@ -1149,6 +1163,10 @@ gst_rtsp_connection_free (GstRTSPConnection * conn)
 
   g_return_val_if_fail (conn != NULL, GST_RTSP_EINVAL);
 
+  if (WRITE_SOCKET (conn) >= 0)
+    CLOSE_SOCKET (WRITE_SOCKET (conn));
+  if (READ_SOCKET (conn) >= 0)
+    CLOSE_SOCKET (READ_SOCKET (conn));
 #ifdef G_OS_WIN32
   WSACleanup ();
 #endif
@@ -1161,6 +1179,103 @@ gst_rtsp_connection_free (GstRTSPConnection * conn)
   return res;
 }
 
+/**
+ * gst_rtsp_connection_poll:
+ * @conn: a #GstRTSPConnection
+ * @events: a bitmask of #GstRTSPEvent flags to check
+ * @revents: location for result flags 
+ * @timeout: a timeout
+ *
+ * Wait up to the specified @timeout for the connection to become available for
+ * at least one of the operations specified in @events. When the function returns
+ * with #GST_RTSP_OK, @revents will contain a bitmask of available operations on
+ * @conn.
+ *
+ * @timeout can be #NULL, in which case this function might block forever.
+ *
+ * This function can be canceled with gst_rtsp_connection_flush().
+ * 
+ * Returns: #GST_RTSP_OK on success.
+ *
+ * Since: 0.10.15
+ */
+GstRTSPResult
+gst_rtsp_connection_poll (GstRTSPConnection * conn, GstRTSPEvent events,
+    GstRTSPEvent * revents, GTimeVal * timeout)
+{
+  fd_set writefds, *pwritefds;
+  fd_set readfds;
+  int max_fd;
+  gint retval;
+  struct timeval tv, *tvp;
+
+  g_return_val_if_fail (conn != NULL, GST_RTSP_EINVAL);
+  g_return_val_if_fail (events != 0, GST_RTSP_EINVAL);
+  g_return_val_if_fail (revents != NULL, GST_RTSP_EINVAL);
+
+  if (events & GST_RTSP_EV_WRITE) {
+    /* add fd to writer set when asked to */
+    FD_ZERO (&writefds);
+    FD_SET (conn->fd, &writefds);
+    pwritefds = &writefds;
+  } else
+    pwritefds = NULL;
+
+  /* always add cancel socket to readfds */
+  FD_ZERO (&readfds);
+  FD_SET (READ_SOCKET (conn), &readfds);
+  if (events & GST_RTSP_EV_READ) {
+    /* add fd to reader set when asked to */
+    FD_SET (conn->fd, &readfds);
+  }
+  max_fd = MAX (conn->fd, READ_SOCKET (conn));
+
+  if (timeout) {
+    tv.tv_sec = timeout->tv_sec;
+    tv.tv_usec = timeout->tv_usec;
+    tvp = &tv;
+  } else
+    tvp = NULL;
+
+  do {
+    retval = select (max_fd + 1, &readfds, pwritefds, NULL, tvp);
+  } while ((retval == -1 && errno == EINTR));
+
+  if (retval == 0)
+    goto select_timeout;
+
+  if (retval == -1)
+    goto select_error;
+
+  if (FD_ISSET (READ_SOCKET (conn), &readfds))
+    goto stopped;
+
+  *revents = 0;
+  if (events & GST_RTSP_EV_READ) {
+    if (FD_ISSET (conn->fd, &readfds))
+      *revents |= GST_RTSP_EV_READ;
+  }
+  if (events & GST_RTSP_EV_WRITE) {
+    if (FD_ISSET (conn->fd, &writefds))
+      *revents |= GST_RTSP_EV_WRITE;
+  }
+  return GST_RTSP_OK;
+
+  /* ERRORS */
+select_timeout:
+  {
+    return GST_RTSP_ETIMEOUT;
+  }
+select_error:
+  {
+    return GST_RTSP_ESYS;
+  }
+stopped:
+  {
+    return GST_RTSP_EINTR;
+  }
+}
+
 /**
  * gst_rtsp_connection_next_timeout:
  * @conn: a #GstRTSPConnection
