@@ -166,8 +166,8 @@ enum
   PROP_SYNCHRONOUS,
   PROP_PIXEL_ASPECT_RATIO,
   PROP_FORCE_ASPECT_RATIO,
-  PROP_HANDLE_EVENTS
-      /* FILL ME */
+  PROP_HANDLE_EVENTS,
+  PROP_HANDLE_EXPOSE
 };
 
 static GstVideoSinkClass *parent_class = NULL;
@@ -180,6 +180,8 @@ static GstVideoSinkClass *parent_class = NULL;
 
 /* ximage buffers */
 
+static GstBufferClass *ximage_buffer_parent_class = NULL;
+
 #define GST_TYPE_XIMAGE_BUFFER (gst_ximage_buffer_get_type())
 
 #define GST_IS_XIMAGE_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_XIMAGE_BUFFER))
@@ -236,6 +238,10 @@ gst_ximage_buffer_finalize (GstXImageBuffer * ximage)
     recycled = TRUE;
   }
 
+  if (!recycled)
+    GST_MINI_OBJECT_CLASS (ximage_buffer_parent_class)->
+        finalize (GST_MINI_OBJECT (ximage));
+
 beach:
   return;
 }
@@ -263,6 +269,8 @@ gst_ximage_buffer_class_init (gpointer g_class, gpointer class_data)
 {
   GstMiniObjectClass *mini_object_class = GST_MINI_OBJECT_CLASS (g_class);
 
+  ximage_buffer_parent_class = g_type_class_peek_parent (g_class);
+
   mini_object_class->finalize = (GstMiniObjectFinalizeFunction)
       gst_ximage_buffer_finalize;
 }
@@ -1035,7 +1043,7 @@ gst_ximagesink_handle_xevents (GstXImageSink * ximagesink)
     }
   }
 
-  if (exposed || configured) {
+  if (ximagesink->handle_expose && (exposed || configured)) {
     g_mutex_unlock (ximagesink->x_lock);
     g_mutex_unlock (ximagesink->flow_lock);
 
@@ -1081,12 +1089,18 @@ gst_ximagesink_event_thread (GstXImageSink * ximagesink)
 {
   g_return_val_if_fail (GST_IS_XIMAGESINK (ximagesink), NULL);
 
+  GST_OBJECT_LOCK (ximagesink);
   while (ximagesink->running) {
+    GST_OBJECT_UNLOCK (ximagesink);
+
     if (ximagesink->xwindow) {
       gst_ximagesink_handle_xevents (ximagesink);
     }
     g_usleep (100000);
+
+    GST_OBJECT_LOCK (ximagesink);
   }
+  GST_OBJECT_UNLOCK (ximagesink);
 
   return NULL;
 }
@@ -1200,6 +1214,7 @@ gst_ximagesink_xcontext_get (GstXImageSink * ximagesink)
   if (!px_formats) {
     XCloseDisplay (xcontext->disp);
     g_mutex_unlock (ximagesink->x_lock);
+    g_free (xcontext->par);
     g_free (xcontext);
     return NULL;
   }
@@ -1271,8 +1286,11 @@ gst_ximagesink_xcontext_get (GstXImageSink * ximagesink)
   g_mutex_unlock (ximagesink->x_lock);
 
   /* Setup our event listening thread */
+  GST_OBJECT_LOCK (ximagesink);
+  ximagesink->running = TRUE;
   ximagesink->event_thread = g_thread_create (
       (GThreadFunc) gst_ximagesink_event_thread, ximagesink, TRUE, NULL);
+  GST_OBJECT_UNLOCK (ximagesink);
 
   return xcontext;
 }
@@ -1300,12 +1318,6 @@ gst_ximagesink_xcontext_clear (GstXImageSink * ximagesink)
 
   GST_OBJECT_UNLOCK (ximagesink);
 
-  /* Wait for our event thread */
-  if (ximagesink->event_thread) {
-    g_thread_join (ximagesink->event_thread);
-    ximagesink->event_thread = NULL;
-  }
-
   gst_caps_unref (xcontext->caps);
   g_free (xcontext->par);
   g_free (ximagesink->par);
@@ -1489,19 +1501,18 @@ gst_ximagesink_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_NULL_TO_READY:
 
       /* Initializing the XContext */
-      if (!ximagesink->xcontext)
+      if (ximagesink->xcontext == NULL) {
         xcontext = gst_ximagesink_xcontext_get (ximagesink);
-
-      GST_OBJECT_LOCK (ximagesink);
-      ximagesink->running = TRUE;
-      if (xcontext)
-        ximagesink->xcontext = xcontext;
-      GST_OBJECT_UNLOCK (ximagesink);
-
-      if (!ximagesink->xcontext) {
-        ret = GST_STATE_CHANGE_FAILURE;
-        goto beach;
+        if (xcontext == NULL) {
+          ret = GST_STATE_CHANGE_FAILURE;
+          goto beach;
+        }
+        GST_OBJECT_LOCK (ximagesink);
+        if (xcontext)
+          ximagesink->xcontext = xcontext;
+        GST_OBJECT_UNLOCK (ximagesink);
       }
+
       /* call XSynchronize with the current value of synchronous */
       GST_DEBUG_OBJECT (ximagesink, "XSynchronize called with %s",
           ximagesink->synchronous ? "TRUE" : "FALSE");
@@ -1740,7 +1751,7 @@ gst_ximagesink_buffer_alloc (GstBaseSink * bsink, guint64 offset, guint size,
         width = result.w;
         height = result.h;
         size = bpp * width * height;
-        GST_DEBUG ("peed pad accepts our desired caps %" GST_PTR_FORMAT
+        GST_DEBUG ("peer pad accepts our desired caps %" GST_PTR_FORMAT
             " buffer size is now %d bytes", desired_caps, size);
       } else {
         GST_DEBUG ("peer pad does not accept our desired caps %" GST_PTR_FORMAT,
@@ -2041,6 +2052,9 @@ gst_ximagesink_set_property (GObject * object, guint prop_id,
       gst_ximagesink_set_event_handling (GST_X_OVERLAY (ximagesink),
           g_value_get_boolean (value));
       break;
+    case PROP_HANDLE_EXPOSE:
+      ximagesink->handle_expose = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -2074,6 +2088,9 @@ gst_ximagesink_get_property (GObject * object, guint prop_id,
     case PROP_HANDLE_EVENTS:
       g_value_set_boolean (value, ximagesink->handle_events);
       break;
+    case PROP_HANDLE_EXPOSE:
+      g_value_set_boolean (value, ximagesink->handle_expose);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -2083,10 +2100,19 @@ gst_ximagesink_get_property (GObject * object, guint prop_id,
 static void
 gst_ximagesink_reset (GstXImageSink * ximagesink)
 {
+  GThread *thread;
+
   GST_OBJECT_LOCK (ximagesink);
   ximagesink->running = FALSE;
+  /* grab thread and mark it as NULL */
+  thread = ximagesink->event_thread;
+  ximagesink->event_thread = NULL;
   GST_OBJECT_UNLOCK (ximagesink);
 
+  /* Wait for our event thread to finish before we clean up our stuff. */
+  if (thread)
+    g_thread_join (thread);
+
   if (ximagesink->ximage) {
     gst_buffer_unref (ximagesink->ximage);
     ximagesink->ximage = NULL;
@@ -2168,6 +2194,7 @@ gst_ximagesink_init (GstXImageSink * ximagesink)
   ximagesink->synchronous = FALSE;
   ximagesink->keep_aspect = FALSE;
   ximagesink->handle_events = TRUE;
+  ximagesink->handle_expose = TRUE;
 }
 
 static void
@@ -2216,6 +2243,10 @@ gst_ximagesink_class_init (GstXImageSinkClass * klass)
       g_param_spec_boolean ("handle-events", "Handle XEvents",
           "When enabled, XEvents will be selected and handled", TRUE,
           G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_HANDLE_EXPOSE,
+      g_param_spec_boolean ("handle-expose", "Handle expose", "When enabled, "
+          "the current frame will always be drawn in response to X Expose "
+          "events", TRUE, G_PARAM_READWRITE));
 
   gstelement_class->change_state = gst_ximagesink_change_state;
 
