@@ -40,6 +40,7 @@
 
 #include <string.h>
 #include <math.h>
+#include <stdlib.h>
 
 G_BEGIN_DECLS
 
@@ -48,10 +49,10 @@ G_BEGIN_DECLS
 #define GST_XVIMAGESINK(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST((obj), GST_TYPE_XVIMAGESINK, GstXvImageSink))
 #define GST_XVIMAGESINK_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass), GST_TYPE_XVIMAGESINK, GstXvImageSink))
+  (G_TYPE_CHECK_CLASS_CAST((klass), GST_TYPE_XVIMAGESINK, GstXvImageSinkClass))
 #define GST_IS_XVIMAGESINK(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj), GST_TYPE_XVIMAGESINK))
-#define GST_IS_XVIMAGESINK_CLASS(obj) \
+#define GST_IS_XVIMAGESINK_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass), GST_TYPE_XVIMAGESINK))
 
 typedef struct _GstXContext GstXContext;
@@ -79,7 +80,7 @@ typedef struct _GstXvImageSinkClass GstXvImageSinkClass;
  * @height: the height in pixels of Display @disp
  * @widthmm: the width in millimeters of Display @disp
  * @heightmm: the height in millimeters of Display @disp
- * @par: the pixel aspect ratio calculated from @width, @widthmm and @height, 
+ * @par: the pixel aspect ratio calculated from @width, @widthmm and @height,
  * @heightmm ratio
  * @use_xshm: used to known wether of not XShm extension is usable or not even
  * if the Extension is present
@@ -115,12 +116,18 @@ struct _GstXContext {
   gboolean use_xshm;
 
   XvPortID xv_port_id;
+  guint nb_adaptors;
+  gchar ** adaptors;
   gint im_format;
 
   GList *formats_list;
   GList *channels_list;
 
   GstCaps *caps;
+
+  /* Optimisation storage for buffer_alloc return */
+  GstCaps *last_caps;
+  gint last_format;
 };
 
 /**
@@ -201,10 +208,11 @@ struct _GstXvImageBuffer {
  * @pool_lock: used to protect the buffer pool
  * @image_pool: a list of #GstXvImageBuffer that could be reused at next buffer
  * allocation call
- * @synchronous: used to store if XSynchronous should be used or not (for 
+ * @synchronous: used to store if XSynchronous should be used or not (for
  * debugging purpose only)
  * @keep_aspect: used to remember if reverse negotiation scaling should respect
  * aspect ratio
+ * @handle_events: used to know if we should handle select XEvents or not
  * @brightness: used to store the user settings for color balance brightness
  * @contrast: used to store the user settings for color balance contrast
  * @hue: used to store the user settings for color balance hue
@@ -220,30 +228,35 @@ struct _GstXvImageSink {
   GstVideoSink videosink;
 
   char *display_name;
+  guint adaptor_no;
 
   GstXContext *xcontext;
   GstXWindow *xwindow;
   GstXvImageBuffer *xvimage;
   GstXvImageBuffer *cur_image;
-  
+
   GThread *event_thread;
   gboolean running;
 
   gint fps_n;
   gint fps_d;
-  
+
   GMutex *x_lock;
   GMutex *flow_lock;
 
   /* object-set pixel aspect ratio */
   GValue *par;
-  
+
   GMutex *pool_lock;
   GSList *image_pool;
-  
+
   gboolean synchronous;
+  gboolean double_buffer;
   gboolean keep_aspect;
-  
+  gboolean draw_border;
+  gboolean handle_events;
+  gboolean handle_expose;
+
   gint brightness;
   gint contrast;
   gint hue;
